#!/usr/bin/env python

"""
Prettyprinted document tree nodes.

Copyright (C) 2017, 2018, 2019, 2023 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.tree.moin import Container

class Node:

    "A simplified tree node representation."

    def __init__(self, name):
        self.name = name
        self.nodes = []

    def __repr__(self):
        return "Node(%r, %r)" % (self.name, self.nodes)

    def visit(self, visitor):
        return visitor.node(self)

    def append(self, node):
        self.nodes.append(node)

    def test(self, other):

        """
        Test whether this node is considered equivalent to 'other', where
        'other' is a moinparser.tree node.

        Return any failing tree nodes or None.
        """

        if other.__class__.__name__ != self.name:
            return self, other, "name"

        if isinstance(other, Container):
            for node, other_node in map(None, self.nodes, other.nodes):
                if node is None or other_node is None:
                    return self, other, node is None and "simple" or "document"
                t = node.test(other_node)
                if t:
                    return t
        elif self.nodes:
            return self, other, "empty"

        return None

# vim: tabstop=4 expandtab shiftwidth=4
